#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# Instance.py
#
# DESCRIPTION:
#
# NOTES:
#

import sys, os, time, dircache
from wxPython.wx import *
from gnue.common import GDebug
from MenuBar import MainMenuBar
from gnue.common import RuntimeSettings
from gnue.common.FileUtils import dyn_import
from gnue.common.events import EventController, Event
from ToolFrame import ToolFrame
from gnue.designer import VERSION, PACKAGE
from gnue.designer.Debugger import DebugSession
import TemplateBase
from TemplateParser import WizardRunner


TITLE=PACKAGE

class BaseInstance(wxFrame, EventController):

  ########################################################################
  #
  # Virtual functions...
  #
  # Override these functions in your Instance
  #

  wizardRunner = WizardRunner

  # Called to load the object from a file buffer/handle
  def loadFile(self, buffer):
    return None

  # Called to load an "empty" object.  Create the root object
  # and any initial child objects (if appropriate)
  def loadEmpty(self):
    return None

  # Called in a tree-like fashion for every object
  # whether loaded in the beginning or later added.
  def inventoryObject(self, object):
    pass

  # Called just before saving the file using GObject's dumpTree
  # You might do some sanity checks, etc.
  def preSave(self):
    return

  # Return an instance of the "primary" visual editor/layout panel
  def createVisualEditor(self):
    return None

  #
  def createTools(self):
    pass


  ########################################################################
  #
  # BaseInstance-specific stuff
  # If you need to overwrite something here,
  # then odds are we really need to provide
  # a hook of some sorts.
  #
  def __init__(self, app, location=None, buffer=None):
    wxFrame.__init__(self, NULL, -1, "")
    EventController.__init__(self)

    # Register ourself with RuntimeSettings
    RuntimeSettings.registerInstance(self)

    # Tell RuntimeSettings that we have information to save
    self.runtime_section = self.properties.nickname + 'Layout'
    RuntimeSettings.registerRuntimeSettingHandler(self, self)

    # And the MRU
    RuntimeSettings.registerRuntimeSettingHandler(self, app.mru)


    self._isdirty = 0
    self._makeBackup = 1
    self._isNew = 1

    self._app = app
    self.connections = app.connections
    self._lastGenericNameSeq = {}
    self.nameMappings = {}

    self._pages = []

    self._path = ""

    if location == None:

      if buffer != None:
        self.__loadFromBuffer(buffer)
        self.makeDirty()
      else:
        self.__createEmptyInstance()

    else:

      if not os.access (location, os.R_OK):
        self.Show(1)
        if wxMessageDialog(self, \
          _('The requested file does not exist.\n') +
          _('Do you want to create this file?') +
          _('\n\nFile: %s') \
            % location, _("File Not Found"), wxYES_NO|wxICON_QUESTION ).ShowModal() == wxID_NO:
          sys.exit()
        self.Show(0)
        self.__createEmptyInstance()
        self._path = location
        self.makeDirty()

      elif not os.access (location, os.W_OK):
        self.Show(1)
        if wxMessageDialog(self, \
          _('The requested file is Read Only.\n') +
          _('To save any changes, you will \n') +
          _('be required to do a "Save As..."\n\nFile: %s') \
            % location, _("Read Only Warning"), wxOK|wxCANCEL|wxICON_EXCLAMATION ).ShowModal() == wxID_CANCEL:
          sys.exit()
        self.Show(0)
        self.__loadFromFile(location)
      else:
        self.__loadFromFile(location)

    self.CreateStatusBar()
    self.SetStatusText('Welcome to GNUe Designer. Watch your step and try not to get hurt.')
    self._menubar = MainMenuBar(self)
    self.SetMenuBar(self._menubar)
    self.Show(true)


    self.rootObject.walk(self.__inventory)


    # Create the supplemental tools
    self.__objectListeners = []
    self.createTools()
    self._menubar.buildMenu()

    # Create our tools
    self.visualEditorWindow = self
    self.visualEditor = self.createVisualEditor()


    self.__objectListeners.append(self.visualEditor)


    self._app.addInstance(self)

    # Fit the widgets to the screen
    self.visualEditor.Fit()
    self.Fit()

    self.SetSize((
       RuntimeSettings.getint(self.runtime_section, 'width', 550),
       RuntimeSettings.getint(self.runtime_section, 'height', 400)))

    self.SetPosition((
       RuntimeSettings.getint(self.runtime_section, 'x', -1),
       RuntimeSettings.getint(self.runtime_section, 'y', -1)))

    self.onSetCurrentObject(self.rootObject, __name__)

    self.Refresh()

    EVT_CLOSE(self, self.OnClose)



  def addTool(self, id, title, baseclass):
    GDebug.printMesg(2,'Adding tool %s [%s]' % (title,id))
    window = ToolFrame(self, title)
    control = baseclass(self, self.rootObject, window)
##    window.finalize()  # Called by MenuBar now :`(
    self.__dict__["%sWindow" % id] = window
    self.__dict__[id] = control
    self.__objectListeners.append(control)


  def addToolCreate(self, title, method):
    self._menubar.addToolCreate(title, method)

  def __loadFromFile(self, location):
    try:
      self._path = location
      fileHandle = open(location,'r')
      self.__loadFromBuffer(fileHandle)
      fileHandle.close()
      self.makeClean()
      self._isNew = 0
    except IOError, msg:
      print "\n%s %s\n\nUnable to open file '%s'. \nUnexpected read error:\n  %s.\n" % (TITLE, VERSION, location, msg)
      sys.exit()
    self._app.mru.addLocation(location)


  def __loadFromBuffer(self, fileHandle):
    self.rootObject = self.loadBuffer(fileHandle)


  def __createEmptyInstance(self):
    self.rootObject = self.loadEmpty()
    self.makeClean()
    self._isNew = 1


  #
  #  Used by RuntimeSettings
  #
  def saveRuntimeSettings(self):
    x, y = self.GetPositionTuple()
    width, height = self.GetSizeTuple()
    return ( self.runtime_section,
             { 'x': abs(x),
               'y': abs(y),
               'height': height,
               'width': width  } )

  # Do we need to be saved?
  def isDirty(self):
    return self._isdirty

  # Mark our form as "dirty" (unsaved changes)
  def makeDirty(self):
    if not self._isdirty:
      self._isdirty = 1
      if self._path == "":
        self.SetTitle( TITLE + _(" - <New %s> *") % self.properties.nickname)
      else:
        self.SetTitle( TITLE + " - " + self._path + " *")
    self.dispatchEvent(Event('MakeDirty'))

  def makeClean(self):
    self._isdirty = 0
    self._isnew = 0
    if self._path == "":
      self.SetTitle( TITLE + _(" - <New %s>") % self.properties.nickname)
    else:
      self.SetTitle( TITLE + " - " + self._path)
    self.dispatchEvent(Event('MakeClean'))


  # Take an object and mangle it all up
  # until it is useful to us
  def __inventory (self, object):

    if object != self.rootObject:
    
      # just a hack for designer.reports.TreeView.py (btami)
      if object._xmlnamespace:
        if object.name:
          object._description = object.name + "(out:)"
        else:
         object._description = object._xmltag + "(out:)"

      # Assign an ID if none exists 
      # TODO: in the future, I want to change these to id=".." instead of name=".."
      if hasattr(object, 'name'):
        if (object.name == None or ( \
            object.name[:3] == "__<" and \
            object.name[-3:] == ">__")):
          object.name = self.getNextGenericName(object._type[2:])
        self.nameMappings[string.lower(object.name)] = object

    # Now, give the tool-specific instances a chance
    self.inventoryObject(object)


  def onSetCurrentObject (self, object, handler):
    self._currentObject = object
    for listener in self.__objectListeners:
      listener.onSetCurrentObject(object, handler)

  def onCreateObject (self, object, handler):
    self.__inventory(object)
    for listener in self.__objectListeners:
      listener.onCreateObject(object, handler)
    self.makeDirty()

  def onModifyObject (self, object, handler, modifications):
    for listener in self.__objectListeners:
      listener.onModifyObject(object, handler, modifications)
    self.makeDirty()

  def onDeleteObject (self, object, handler):

    # Notify all listeners
    for listener in self.__objectListeners:
      listener.onDeleteObject(object, handler)

    # Delete the actual object from its parent
    i = 0
    while i < len(object._parent._children) and \
       object._parent._children[i] != object:
      i = i + 1
    if object._parent._children[i] == object:
      object._parent._children.pop(i)

    self.makeDirty()


  def getNextGenericName (self, type):
    while 1:
      if self._lastGenericNameSeq.has_key(string.lower(type)):
        self._lastGenericNameSeq[string.lower(type)] = \
           self._lastGenericNameSeq[string.lower(type)] + 1
      else:
        self._lastGenericNameSeq[string.lower(type)] = 1
      name = "%s%s_%s" % (string.upper(type[0]), \
          string.lower(type[1:]), self._lastGenericNameSeq[string.lower(type)])
      if not self.nameMappings.has_key(string.lower(name)):
        break

    return name


  def getUniqueName(self, n):
    name = string.lower(n)
    while self.nameMappings.has_key(name):
      if '_' in name:
        nameparts = string.split(name,'_')
        try:
          nameparts[-1] = str(int(nameparts[-1]) + 1)
          name = string.join(nameparts,'_')
        except ValueError:
          name += '_1'
      else:
        name += '_1'
    return name


  def save(self):

    self.preSave()

    location = self._path
    fileHandle = None
    fileHandle2 = None
    if self._makeBackup:
      try:
        fileHandle = open(location,'r')
        fileHandle2 = open(location + "~",'w')
        fileHandle2.writelines(fileHandle.readlines())
      except:
        pass
      else:
        if fileHandle != None:
          fileHandle.close()
        if fileHandle2 != None:
          fileHandle2.close()

    self._makeBackup = 0

    fileHandle = open(location,'w')
    if sys.getdefaultencoding() == 'ascii':
      fileHandle.write('<?xml version="1.0"?>\n\n')
    else:
      fileHandle.write('<?xml version="1.0" encoding="%s"?>\n\n' % \
           (sys.getdefaultencoding()))

    fileHandle.write('<!--  %s (%s)\n      Saved on: %s  -->\n\n' \
       % (TITLE, VERSION, \
          time.strftime("%Y-%m-%d %H:%M:%S",time.localtime(time.time()))))
    fileHandle.write(self.rootObject.dumpXML(treeDump=1))
    fileHandle.close()

    self.makeClean()



  def OnConnectTo(self, event):
    # connectionMap is built by MenuBar
    conn = self.connectionMap[event.GetId()]
    tempDO = self.connections.getDataObject(conn, 'object')
    try:
      self.connections.requestConnection(tempDO, conn)
      self._menubar._fileConnect.Enable(event.GetId(),0)
      self.onSetCurrentObject(self._currentObject, __name__)
    except:
      print _("Unable to connect to %s") % conn


  def OnNew(self, event):
    self._app.newInstance(self._fileNewMap[event.GetId()])


  def OnSave(self, event):
    if not len(self._path):
      self.OnSaveAs(event)
    else:
      self.save()


  def OnSaveAs(self, event):
    wildcard = ""
    # Make the "default" file extension for a tool
    # appear before the other extensions.
    filterIndex = [self.properties.defaultFileExtension]
    wildcard += "%s (*.%s)|*.%s|" % \
     ( self.properties.fileExtensions[self.properties.defaultFileExtension],
       self.properties.defaultFileExtension,
       self.properties.defaultFileExtension)

    for type in self.properties.fileExtensions.keys():
      if type != self.properties.defaultFileExtension:
        wildcard += "%s (*.%s)|*.%s|" % \
           ( self.properties.fileExtensions[type], type, type)
        filterIndex.append(type)

    dlg = wxFileDialog(NULL, _("Save %s As...") % self.properties.description, defaultDir=os.getcwd(),
                           wildcard = wildcard,
                           style=wxSAVE)

    if dlg.ShowModal() == wxID_OK:
      self._path = dlg.GetPath()
      if len(self._path) < 4 or not (self._path[-4] == '.' and string.lower(self._path[-3:]) in self.properties.fileExtensions.keys()):
        self._path += "." + filterIndex[dlg.GetFilterIndex()]
      self.SetTitle (TITLE + " - " + self._path)
      self.save()

    dlg.Destroy()


  def OnClose(self, event):
    if self.isDirty():
      dlg = wxMessageDialog(NULL,
              _("This document has unsaved changes.\n") +
              _("Save changes before closing?"),
              _("Unsaved Changes"), style=wxYES_NO|wxCANCEL|wxICON_WARNING)
      save = dlg.ShowModal()
      dlg.Destroy()
      if save == wxID_YES:
        self.OnSave(event)
      elif save == wxID_CANCEL:
        event.Veto()
        return

    RuntimeSettings.saveRuntimeSettings(self)
    self._app.removeInstance(self)
    self.Destroy()

  def OnDebugRun (self, event):
    tmpLevel = 0
    tmpRez = 0
    tmpMessage = _("Please enter debugging level - integer [0..n]")
    tmpPrompt = _("Level:")
    tmpCaption = _("Choose debug level")
    tmpRez = wxGetNumberFromUser(tmpMessage,tmpPrompt,tmpCaption,tmpLevel,0,10,NULL)
    if (tmpRez == -1):
      tmpMessage = _("Debugging has been cancelled.")
      tmpCaption = _("Debugging...")
      wxMessageBox(tmpMessage,tmpCaption,wxOK|wxCENTRE,NULL,-1,-1)
    else:
      GDebug.setDebug(tmpRez,"");
      DebugSession(self)



  #################################################################
  #
  #################################################################

  def loadWizards(self, package):
    templates = []

############################
# This is a workaround for McMillan packaging!!!
# gnue.designer.WizardFiles.py
# is autocreated at McMillan's analysis time by
# hook-gnue.designer.Designer.py
# It contains a list of available wizards.
# McMillan sets sys.frozen=1 in packaged exe file.
############################
    if hasattr(sys, 'frozen'):
      from gnue.designer import WizardFiles
      for template in WizardFiles.WizardFiles:
        t = template.split('.')
        if '.'.join(t[0:4]) == package.__name__:
          try:
            templates.append(dyn_import('%s' % template).TemplateInformation)
          except ImportError:
            pass
          except AttributeError:
            pass
    else:
############
# end of workaround
############

      basedir = os.path.dirname(package.__file__)
      processed = []  # Base file names processed (e.g., base of Simple.py*
                    # is Simple) This will keep us from importing Simple
                    # three times if Simple.py, Simple.pyc, and Simple.lib
                    # all exist.

      for dir in dircache.listdir(basedir):
        base = string.split(dir,'.')[0]
        if not dir[0] in ('.','_') and not base in processed:
          processed.append(base)
          try:
            templates.append(dyn_import(
               '%s.%s' % (package.__name__,base)).TemplateInformation)
          except ImportError, mesg:
            GDebug.printMesg(2,"%s.%s doesn't appear to be a valid wizard" % (package.__name__, base))
            GDebug.printMesg(5,' --> %s' % (mesg))
          except AttributeError:
            GDebug.printMesg(2,'Wizard %s for package %s is missing'
                     ' an \'TemplateInformation\' attribute.' %
                             (base,package.__name__))

    for template in templates:
      try:
        location = template['Location']
        if location:
          self._menubar.addWizardLocation(location, template)
      except ValueError:
        continue

#
# Helper class used by the tool-specific packages
#
class ToolProperties:
  xmlOpeningTag = 'undefined'
  short = 'undefined'
  application = 'GNUe Tool'
  description = 'undefined'
  fileExtensions = {}
  defaultFileExtension = 'undefined'











