#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# PropertyEditor.py
#
# DESCRIPTION:
#
# NOTES:
#


import sys, os, string
from wxPython.wx import *
from wxPython.grid import *
from gnue.common import GDebug, GTypecast
from ToolFrame import ToolFrame

class PropertyEditor (wxPanel):
  def __init__(self, instance, root, parent):
    wxPanel.__init__(self,parent,-1,pos=wxPoint(0,0))
    self.panel = wxPanel(self,-1,style=wxSIMPLE_BORDER, pos=wxPoint(6,6))

    self.parent = parent
    self.instance = instance
    self.rootObject = root
    self.object = None
    self.objectList = []
    self.objectMap = {}

    # Set up our grid
    self.grid = wxGrid(self.panel, -1, pos=wxPoint(0,0))
    self.grid.CreateGrid(1,1)
    self.grid.SetColLabelSize(0)

    EVT_GRID_CELL_CHANGE(self.grid, self.OnCellChange)
    EVT_GRID_CELL_LEFT_DCLICK(self.grid, self.OnLeftDClick)
    EVT_SIZE(self, self.onSize)


  def onSetCurrentObject (self, object, handler):
    if object == None:
      return
    if handler != __name__:
      self.object = object
      elements = self.instance.incubator.elements
      try:
        self.attributes = elements[string.lower(object._type[2:])]['Attributes']
      except KeyError:
        self.attributes = {}

      self.elements = elements[string.lower(object._type[2:])]
 
      # Speed up the process by not refreshing the grid yet
      self.grid.BeginBatch()


      # Delete any old rows from a previous object
      nr = self.grid.GetNumberRows()
      if nr:
        self.grid.DeleteRows(0,nr)


      self.rowList = self.attributes.keys()
      self.rowList.sort()


      # Only show properties for nondeprecated values (unless set)
      i = 0
      while i < len(self.rowList):
        key = self.rowList[i]
        if self.attributes[key].has_key ('Deprecated') and \
           self.attributes[key]['Deprecated'] and \
           (not hasattr(object, key) or \
              (self.attributes[key].has_key('Default') and \
               object.__dict__[key] == self.attributes[key]['Default'] ) ): 
          self.rowList.pop(i)
        else: 
          i = i + 1


      # Create the number of rows we'll need
      self.grid.InsertRows(0,len(self.rowList))

      i = 0

      for key in self.rowList:
  
        # This little tidbit does mixed case w/'_' as separators
        words = string.split(key,'_')
        for j in range(len(words)): 
          try:
            words[j] = string.upper(words[j][0]) + string.lower(words[j][1:])
          except IndexError: 
            words[j] = string.upper(words[j])
        self.grid.SetRowLabelValue(i,string.join(words,' '))


        # Determine the type of Cell Editor we want 
        # (Integer, Boolean, Dropdown, Char)
        if self.attributes[key].has_key('ValueSet'): 
          self.grid.SetCellEditor(i, 0, RestrictedCellEditor(self.grid,self.attributes[key]))
        elif self.attributes[key]['Typecast'] == GTypecast.boolean: 
          self.grid.SetCellEditor(i, 0, BoolCellEditor(self.grid,self.attributes[key]))
        elif self.attributes[key]['Typecast'] in (GTypecast.integer,
                GTypecast.number, GTypecast.whole):
          self.grid.SetCellEditor(i, 0, IntCellEditor(self.grid,self.attributes[key]))
        else: 
           self.grid.SetCellEditor(i, 0, CharCellEditor(self.grid,self.attributes[key]))


        # Set the initial value of the cells to the current property values 
        if hasattr(object, key): 
          if self.attributes[key]['Typecast'] == GTypecast.boolean:
            if object.__dict__[key]:    
              self.grid.SetCellValue(i,0,"TRUE")
            else: 
              self.grid.SetCellValue(i,0,"FALSE")
              pass
          else: 
            self.grid.SetCellValue(i,0, "%s" % object.__dict__[key])
            pass
        i = i + 1


      # Redraw the grid
      self.grid.EndBatch()
      try: 
        self.grid.ForceRefresh()  # Added in 2.3.1
      except AttributeError: 
        pass


  def onCreateObject (self, object, handler):
    pass


  def onModifyObject (self, object, handler, modifications):
    if object == None: 
      return
    if handler != __name__ and self.object == object:
#      for modification in modifications: 
#        attribute, value = modification
      self.onSetCurrentObject(object, handler)


  def onDeleteObject (self, object, handler):
    if object == None: 
      return
    if handler != __name__:
      pass


  def onSize(self, event):
    x,y = self.GetClientSizeTuple()
    self.panel.SetSize((x-12,y-12))
    self.grid.SetSize(self.panel.GetClientSizeTuple())
    w,h = self.grid.GetClientSizeTuple()
    self.grid.SetColSize(0, w - self.grid.GetRowLabelSize())
    try:
      self.grid.ForceRefresh()  # Added in 2.3.1
    except AttributeError:
      pass



  # Force a grid cell into Edit mode when Double-Clicked
  def OnLeftDClick(self,evt): 
    if self.grid.CanEnableCellControl(): 
      self.grid.EnableCellEditControl()

  def OnCellChange(self, evt):
    attr = self.rowList[evt.GetRow()]
    value = self.grid.GetCellValue(evt.GetRow(), evt.GetCol())

    try: 
      self.object.__dict__[attr] = self.attributes[attr]['Typecast'](value)
      self.instance.onModifyObject(self.object, __name__, ((attr, self.object.__dict__[attr]),))
      evt.Skip()
    except ValueError: 
      wxBell()

#
#
#
class CharCellEditor (wxPyGridCellEditor):
  def __init__(self, grid, attributes): 
    wxPyGridCellEditor.__init__(self)
    self.grid = grid
    self.attributes = attributes
    self.__created = 0


  def Create(self, parent, id, evtHandler):
    self.__created = 1
    GDebug.printMesg(10,'Creating CharCellEditor')
    self._tc = wxTextCtrl(parent, id, "")
    self._tc.SetInsertionPoint(0)
    self.SetControl(self._tc)
    if evtHandler:
      self._tc.PushEventHandler(evtHandler)

  def SetSize(self, rect):
    self._tc.SetDimensions(rect.x, rect.y, rect.width+2, rect.height+2)

  def Show(self, show, attr):
    self.base_Show(show, attr)

  def PaintBackground(self, rect, attr):
    pass

  def BeginEdit(self, row, col, grid):
    self.startValue = grid.GetTable().GetValue(row, col)
    self._tc.SetValue(self.startValue)
    self._tc.SetInsertionPointEnd()
    self._tc.SetFocus()

    # For this example, select the text
    self._tc.SetSelection(0, self._tc.GetLastPosition())


  def EndEdit(self, row, col, grid):
    changed = false

    val = self._tc.GetValue()
    if val != self.startValue:
       changed = true
       grid.GetTable().SetValue(row, col, val) # update the table

    self.startValue = ''
    self._tc.SetValue('')
    return changed


  def Reset(self):
    self._tc.SetValue(self.startValue)
    self._tc.SetInsertionPointEnd()


  def IsAcceptedKey(self, evt):
    return (not (evt.ControlDown() or evt.AltDown()) and
            evt.GetKeyCode() != WXK_SHIFT)


  def StartingKey(self, evt):
    GDebug.printMesg(10, "Starting Key; created=%s" % self.__created)
    key = evt.GetKeyCode()
    ch = None
    if key in [WXK_NUMPAD0, WXK_NUMPAD1, WXK_NUMPAD2, WXK_NUMPAD3, WXK_NUMPAD4,
               WXK_NUMPAD5, WXK_NUMPAD6, WXK_NUMPAD7, WXK_NUMPAD8, WXK_NUMPAD9]:
        ch = ch = chr(ord('0') + key - WXK_NUMPAD0)

    elif key < 256 and key >= 0 and chr(key):
      ch = chr(key)
      if not evt.ShiftDown():
        ch = string.lower(ch)

    if ch is not None:
      # For this example, replace the text.  Normally we would append it.
      #self._tc.AppendText(ch)
#      self._tc.SetValue(ch)
 #     self._tc.SetInsertionPointEnd()
      pass
    else:
      evt.Skip()


  def StartingClick(self):
    pass

  def Destroy(self):
    self.base_Destroy()

  def Clone(self): 
    return CharCellEditor(self.attributes)


class IntCellEditor (CharCellEditor):
  def __init__(self, grid, attributes): 
    CharCellEditor.__init__(self, grid, attributes)

  def Create(self, parent, id, evtHandler):
    CharCellEditor.Create(self, parent, id, evtHandler)
    EVT_CHAR(self._tc, self.OnKeyPressed)

  def OnKeyPressed(self, evt): 
    if (ord('0') <= evt.KeyCode() <= ord('9') or \
        evt.KeyCode() in (ord('-'), ord('+')) or \
        evt.KeyCode() < 32 or evt.KeyCode() > 126): 
      evt.Skip()

  def StartingKey(self, evt):
    key = evt.GetKeyCode()
    ch = None
    if key in [WXK_NUMPAD0, WXK_NUMPAD1, WXK_NUMPAD2, WXK_NUMPAD3, WXK_NUMPAD4,
               WXK_NUMPAD5, WXK_NUMPAD6, WXK_NUMPAD7, WXK_NUMPAD8, WXK_NUMPAD9]:
        ch = ch = chr(ord('0') + key - WXK_NUMPAD0)

    elif key < 256 and key >= 0 and chr(key):
      ch = chr(key)
      if not evt.ShiftDown():
        ch = string.lower(ch)

    if '0' <= ch <= '9':
      # For this example, replace the text.  Normally we would append it.
      #self._tc.AppendText(ch)
      pass
#      self._tc.SetValue(ch)
#      self._tc.SetInsertionPointEnd()
    else:
      evt.Skip()


class RestrictedCellEditor (CharCellEditor):
  def __init__(self, grid, attributes): 
    CharCellEditor.__init__(self, grid, attributes)
    self.valueList = attributes['ValueSet'].keys()
      
    self.valueList.sort()
    self.selectionList = []
    self.valueMap = {}

    i = 0 
    if not (attributes.has_key('Required') and attributes['Required'] ): 
      self.valueMap[''] = 0
      self.selectionList.append(' <Not Set> ')
      i = 1

    for v in self.valueList: 
      self.valueMap[v] = i
      self.selectionList.append("%s" % (attributes['ValueSet'][v] or v))
      i = i + 1

    self.valueList.insert(0, '')

  def Create(self, parent, id, evtHandler):
    GDebug.printMesg(10,'Creating CharCellEditor')
    self._tc = wxComboBox(parent, id, "", style=wxCB_READONLY, 
                          choices=self.selectionList)
    self.SetControl(self._tc)
    if evtHandler:
      self._tc.PushEventHandler(evtHandler)

  def BeginEdit(self, row, col, grid):
    self.startValue = grid.GetTable().GetValue(row, col)
    self._tc.SetSelection(self.valueMap[self.startValue])
    self._tc.SetFocus()

  def EndEdit(self, row, col, grid):
    changed = false

    val = self.valueList[self._tc.GetSelection()]
    if val != self.startValue:
       changed = true
       grid.GetTable().SetValue(row, col, "%s" % val) # update the table

    self.startValue = ''
    self._tc.SetSelection(0)
    return changed


  def Reset(self):
    self._tc.SetSelection(self.valueMap[self.startValue])
    #self._tc.SetInsertionPointEnd()


  def StartingKey(self, evt):
    key = evt.GetKeyCode()
    ch = None
    if key in [WXK_NUMPAD0, WXK_NUMPAD1, WXK_NUMPAD2, WXK_NUMPAD3, WXK_NUMPAD4,
               WXK_NUMPAD5, WXK_NUMPAD6, WXK_NUMPAD7, WXK_NUMPAD8, WXK_NUMPAD9]:
        ch = ch = chr(ord('0') + key - WXK_NUMPAD0)

    elif key < 256 and key >= 0 and chr(key):
      ch = chr(key)
      if not evt.ShiftDown():
        ch = string.lower(ch)

    if ch is not None:
      # For this example, replace the text.  Normally we would append it.
      #self._tc.AppendText(ch)
      pass
#      self._tc.SetValue(ch)
#      self._tc.SetInsertionPointEnd()
    else:
      evt.Skip()


class BoolCellEditor (CharCellEditor):
  def __init__(self, grid, attributes):
    CharCellEditor.__init__(self, grid, attributes)
      
    self.selectionList = ['FALSE','TRUE']

  def Create(self, parent, id, evtHandler):
    self._tc = wxComboBox(parent, id, "", style=wxCB_READONLY, 
                          choices=self.selectionList)
    self.SetControl(self._tc)
    if evtHandler:
      self._tc.PushEventHandler(evtHandler)

  def BeginEdit(self, row, col, grid):
    self.startValue = grid.GetTable().GetValue(row, col)
    
    self._tc.SetValue(self.startValue)
    self._tc.SetFocus()

  def EndEdit(self, row, col, grid):
    changed = false

    if self._tc.GetSelection(): 
      val = 'TRUE' 
    else: 
      val = 'FALSE' 
    if val != self.startValue:
       changed = true
       grid.GetTable().SetValue(row, col, "%s" % val) # update the table

    self.startValue = ''
    self._tc.SetSelection(0)
    return changed


  def Reset(self):
    self._tc.SetValue(self.startValue)
    #self._tc.SetInsertionPointEnd()


  def StartingKey(self, evt):
    key = evt.GetKeyCode()
    ch = None
    if key in [WXK_NUMPAD0, WXK_NUMPAD1, WXK_NUMPAD2, WXK_NUMPAD3, WXK_NUMPAD4,
               WXK_NUMPAD5, WXK_NUMPAD6, WXK_NUMPAD7, WXK_NUMPAD8, WXK_NUMPAD9]:
        ch = ch = chr(ord('0') + key - WXK_NUMPAD0)

    elif key < 256 and key >= 0 and chr(key):
      ch = chr(key)
      if not evt.ShiftDown():
        ch = string.lower(ch)

    if ch is not None:
      # For this example, replace the text.  Normally we would append it.
      #self._tc.AppendText(ch)
      pass
#      self._tc.SetValue(ch)
#      self._tc.SetInsertionPointEnd()
    else:
      evt.Skip()


#
#
#
class EventEditor (wxPanel): 
  def __init__(self, instance, root, parent):
    wxPanel.__init__(self,parent,-1,pos=wxPoint(0,0))
    self.panel = wxPanel(self,-1,style=wxSIMPLE_BORDER, pos=wxPoint(6,6))

    self.parent = parent
    self.instance = instance
    self.rootObject = root
    self.objectList = []
    self.objectMap = {}
    self.object = None
    self.grid = wxGrid(self.panel, -1, pos=wxPoint(0,0))
    self.grid.CreateGrid(1,1)
    EVT_SIZE(self, self.onSize)

  def onSetCurrentObject (self, object, handler):

    if object == None: 
      return
    if handler != __name__:
      self.object = object
      elements = self.instance.incubator.elements
      try:
        self.attributes = elements[string.lower(object._type[2:])]['Attributes']
      except KeyError:
        self.attributes = {}

      self.elements = elements[string.lower(object._type[2:])]

      # This is a nasty temporary way of refreshing grid
      if self.grid: 
        self.grid.Destroy()
      self.grid = wxGrid(self.panel, -1, pos=wxPoint(0,0))

      self.grid.CreateGrid(len(self.attributes.keys()),1)
      self.grid.SetColLabelSize(0)
      self.rowList = self.attributes.keys()
      self.rowList.sort()
      i = 0
      for key in self.rowList: 
        self.grid.SetRowLabelValue(i, string.upper(key[0]) + string.lower(key[1:]))
        if hasattr(object, key): 
          self.grid.SetCellValue(i,0,"%s" % object.__dict__[key])
        i = i + 1
      self.onSize(None)
      EVT_GRID_CELL_CHANGE(self.grid, self.OnCellChange)

  def onCreateObject (self, object, handler):

    # Until this works!
    return

    pass

  def onModifyObject (self, object, handler, modifications):

    # Until this works!
    return

    if object == None: 
      return
    if handler != __name__ and self.object == object:
#      for modification in modifications: 
#        attribute, value = modification
      self.onSetCurrentObject(object, handler)

  def onDeleteObject (self, object, handler):

    # Until this works!
    return

    if object == None: 
      return
    if handler != __name__:
      pass

  def onSize(self, event): 
    x,y = self.GetClientSizeTuple()
    self.panel.SetSize((x-12,y-12))
    self.grid.SetSize(self.panel.GetClientSizeTuple())
    w,h = self.grid.GetClientSizeTuple()
    self.grid.SetColSize(0, w - self.grid.GetRowLabelSize())

  def OnCellChange(self, evt):
    attr = self.rowList[evt.GetRow()]
    value = self.grid.GetCellValue(evt.GetRow(), evt.GetCol())

    try: 
      self.object.__dict__[attr] = self.attributes[attr][2](value)
      self.instance.onModifyObject(self.object, __name__, ((attr, self.object.__dict__[attr]),))
      evt.Skip()
    except ValueError: 
      wxBell()
      
#
#
#
class PropertyToolFrame(ToolFrame):
  def __init__(self, instance, title): 
    ToolFrame.__init__(self, instance, _("Property Editor"))

    self.backpanel = wxPanel(self, -1, pos=wxPoint(0,0), size=self.GetClientSize())
#    self.backpanel = self
    self.toolpanel = wxPanel(self.backpanel, -1, pos=wxPoint(0,0), size=wxSize(32,32), 
         style=wxRAISED_BORDER|wxCLIP_CHILDREN)

    self.notebook = wxNotebook(self.backpanel, -1, pos=wxPoint(6, 37), size=wxSize(32,32))

    self.propertyEditor = PropertyEditor(
         self.notebook, instance, instance.rootObject)

    self.eventEditor = EventEditor(
         self.notebook, instance, instance.rootObject)

    self.notebook.AddPage(self.propertyEditor, _('Properties'))
    self.notebook.AddPage(self.eventEditor, _('Events'))

    self.objectLabel = wxStaticText(self.toolpanel, -1, _("Object: "), 
         pos=wxPoint(4,6))
    self.objectCombo = wxComboBox(self.toolpanel, -1, 
         pos=wxPoint(self.objectLabel.GetSize().width + 10,4), 
         style=wxCB_READONLY)

    EVT_COMBOBOX(self, self.objectCombo.GetId(), self.OnObjectSelected)

    EVT_SIZE(self, self.onSize)

    self.objectList = [instance.rootObject]

    instance.rootObject.walk (self.inventoryObject)
    self.generateObjectMap()

  def generateObjectMap(self): 
    self.objectMap = {}

    # Alphabetize the combo list
    nameMap = {}
    for o in self.objectList: 
      nameMap[string.lower(o.name)] = o
    keys = nameMap.keys()
    keys.sort()   
    i = 0

    self.objectList = []
    self.objectCombo.Clear()
    for key in keys: 
      self.objectList.append(nameMap[key])
      self.objectMap[nameMap[key]] = i
      self.objectCombo.Append(nameMap[key].name)
      i = i + 1

  def inventoryObject(self, object):
    if hasattr(object, 'name'):
      self.objectList.append(object)

  def OnObjectSelected(self, event): 
    self.instance.onSetCurrentObject(
       self.objectList[event.GetSelection()], "%sFrame" % __name__)

  def onSize(self, event):
    w,h = self.GetClientSizeTuple()
    self.backpanel.SetSize(wxSize(w,h))
    self.toolpanel.SetSize(wxSize(w,32))
    self.notebook.SetSize(wxSize(w-12,h-44))

  def onSetCurrentObject (self, object, handler):
    if object == None: 
      return
    if handler != "%sFrame" % __name__:
      if self.objectMap.has_key(object): 
        self.objectCombo.SetSelection(self.objectMap[object])

  def onCreateObject (self, object, handler):
    if object == None: 
      return
    self.inventoryObject(object)
    self.generateObjectMap()

  def onModifyObject (self, object, handler, modifications):
    if object == None: 
      return

    # TODO: This should see if object is listed in 
    # TODO: the pull-down box. If so, monitor for 
    # TODO: name change.

  def onDeleteObject (self, object, handler):
    if object == None: 
      return
    self.objectList.pop(self.objectMap[object])
    self.generateObjectMap()

