#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# Incubator.py
#
# DESCRIPTION:
# Creates objects to be added to the object
#
# NOTES:
#

from gnue.common import GDebug, GParserHelpers
from gnue.forms import GFParser, GFForm
from gnue.designer.Incubator import *

def createObject(instance, form, tag, type=None, parent=None,
         parentHint=None, attributes={}):


  GDebug.printMesg(3,'Creating a "%s" object' % tag)
  GDebug.printMesg(6,"Object's parent is %s" % parentHint)

  try:
    name = attributes['name']
  except KeyError:
    name = instance.getNextGenericName(tag)

  try:
    objclass = elements[tag]['BaseClass']
    if elements[tag].has_key('Attributes'):
      attrs = elements[tag]['Attributes']
    else:
      attrs = {}
  except KeyError:
    GDebug.printMesg(1, "Attempted to create a '%s', "
                        "but I don't know what to do!" % tag)
    return


  if tag == 'trigger':
    if not attributes.has_key('type'):
      attributes['type'] = 'NAMED'
    if attributes['type'] == 'NAMED':
      parent = form
    else:
      parent = parent or parentHint
  elif tag in ('page','datasource','import'):
    parent = form

  o = objclass(parent or parentHint)

  # Add any GContent
  try:
    GParserHelpers.GContent(o, attributes['_content_'])
  except KeyError:
    pass

  # Pull default values for any attributes not supplied
  for attr in attrs.keys():
    if not attributes.has_key(attr):
      if attrs[attr].has_key('Default'):
        attributes[attr] = attrs[attr]['Typecast'](attrs[attr]['Default'])
      elif attr in ('x','y',):
        attributes[attr] = 0
      elif attr in ('height',):
        if tag in ('box','scrollbar'):
          attributes[attr] = 4
        else:
          attributes[attr] = 1
      elif attr == 'width':
        if tag in ('scrollbar',):
          attributes[attr] = 1
        else:
          attributes[attr] = 10
      elif attr in ('text','label'):
        attributes[attr] = name

  for attr in attributes.keys():
    if attrs.has_key(attr):
      o.__dict__[attr] = attributes[attr]

  o.name = name
  instance.nameMappings[o.name] = o
  o._buildObject()
  instance.onCreateObject(o, __name__)
  instance.onSetCurrentObject(o, __name__)
  return o


def reparentObject(instance, form, object, newParent):
  if object._parent == newParent:
    return

  # TODO
  pass


def deleteObject(instance, form, object, newCurrentObject=None, firstRun=1):

  if firstRun:
    newCurrentObject = object._parent
    parent = object._parent

  for child in object._children:
    child.deleteObject(instance, form, child, firstRun=0)

  instance.onDeleteObject(object, __name__)

  if firstRun:
    o = parent
    while not isinstance(o, GFForm.GFForm):
      o._buildObject()
      o = o._parent

    if newCurrentObject:
      instance.onSetCurrentObject(newCurrentObject, __name__)



#
# Figure out tag dependencies
#
elements = GFParser.getXMLelements()
elementMapping = {}
calcDependencies(elements, elementMapping)

