#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2001-2002 Free Software Foundation
#
# FILE:
# DisplayDropTarget.py
#
# DESCRIPTION:
#
# NOTES:

from wxPython.wx import *
import sys, string
from Utils import *
from gnue.common import GDebug
from gnue.designer.forms import Incubator

import cPickle

#
# Drag and Drop support for our grid
#
class DisplayDropTarget(wxPyDropTarget):

  def __init__(self, editor):
    wxPyDropTarget.__init__(self)
    self.editor = editor

    self.data = wxCustomDataObject(wxCustomDataFormat("GNUeDesVisualElement"))
    self.SetDataObject(self.data)

  def OnEnter(self, x, y, d):
    return d

  def OnLeave(self):
    pass

  def OnDrop(self, x, y):
    pass

  def OnDragOver(self, x, y, d):
    return d

  # Called when OnDrop returns true.  We need to get the data and
  # do something with it.
  def OnData(self, x, y, d):

    # copy the data from the drag source to our data object
    if self.GetData():
      # convert it back to our format
      data = self.data.GetData()
      unpickled = cPickle.loads(data)


      nextY = int(y / self.editor.gridHeight)

      for params in unpickled:
        x = int(x / self.editor.gridWidth)
        y = nextY

        attributes = params['Attributes']

        # Entry types require a "block".. find or create one
        if params['Type'] in ('entry',):
          try:
            parent = params['Block']
          except KeyError:
            # Find the parent block

            parent = None

            if  not attributes.has_key('block') and \
                attributes.has_key('datasource'):

              datasource = string.lower(attributes['datasource'])

              for child in self.editor.page._children:
                if child._type == 'GFBlock' and \
                  hasattr(child,'datasource') and \
                  string.lower(child.datasource) == datasource:
                  parent = child
                  break

              if parent is None:
                dlg = wxMessageDialog(NULL,
                  "The current page does not have a block\n"
                  "corresponding to Data Source %s.\n"
                  "Create a new block on the current page?" % \
                       attributes['datasource'],
                  "No Target Block Found", style=wxYES_NO|wxICON_WARNING)
                save = dlg.ShowModal()
                dlg.Destroy()
                if save == wxID_NO:
                  return d # TODO: This should actually return a failure

                # Create a parent block
                parent = Incubator.createObject(
                            self.editor._instance,
                            self.editor._instance.rootObject,
                            'block',
                            parent=self.editor.page,
                            attributes={'datasource':datasource})
            else:
              for child in self.editor.page._children:
                if child._type == 'GFBlock' and \
                   child.name == attributes['block']:

                  parent = child
                  break

          try:
            defaulttype = params['DefaultType']
            defaultval = params['DefaultVal']
            if defaulttype == 'constant':
              if defaultval[:1] in "\"'":
                defaultval = defaultval [1:-2]
              attributes['default'] = defaultval
          except KeyError:
            pass



          createLabels = "left" # TODO: string.lower(gConfig("CreateLabelsOnDrop"))

          if createLabels != "no":
            label = params["Label"] + ":"

            # Figure out the position of the label
            # If the entry is for a multirow block,
            # we assume that the label goes above.
            if createLabels == "left" and\
               not(hasattr(parent,'rows') and parent.rows > 1 or 0):
              lx = x - len(label) - 1
              if lx < 0:
                x = len(label) + 1
                lx = 0
              ly = y
            else:  # "above"
              nextY += 1
              lx = x
              ly = y - 1

              if ly < 0:
                ly = 0
                y += 1

            # Create a label
            Incubator.createObject(
                    self.editor._instance,
                    self.editor._instance.rootObject,
                    'label',
                    parent=self.editor.page,
                    attributes={'x': lx,
                                'y': ly,
                                'width': len(label),
                                'text': label })
          nextY += 1

        elif params['Type'] in ('datasource',):
          parent = self.editor.rootObject

        elif params['Type'] in ('block',):
          parent = self.editor.page
        elif params['Type'] in ('selectedWidgets',):
          xshift = x - int(attributes['startingX'])
          yshift = y - int(attributes['startingY'])
          for uiwidget in self.editor._currentSelection.keys():
            uiwidget.relativeMove(xshift,yshift)
        else:
          parent = self.editor.page
          nextY += 1

        attributes['x'] = x
        attributes['y'] = y


        #
        # Create our new object
        #
        if not params['Type'] in ('selectedWidgets',):
          object = Incubator.createObject(
            self.editor._instance,
            self.editor._instance.rootObject,
            params['Type'],
            parent=parent,
            attributes=attributes)

          if params['Type'] in ('entry',):
            try:
              defaulttype = params['DefaultTrigger']
              defaultval = params['DefaultInfo']
              triggertext = ""
              if defaulttype == 'sequence':
                triggertext = "\n# Set default value\n" \
                            + "if self.isEmpty():\n" \
                            + "  self.autofillBySequence('%s')\n" % defaultval
              if defaulttype == 'system' and defaultval == 'timestamp':
                triggertext = "\n# Set default value\n" \
                            + "if self.isEmpty():\n" \
                            + "  self.value = %s.extensions.getTimeStamp()\n" \
                              % parent.datasource


              if triggertext:
                Incubator.createObject(
                    self.editor._instance,
                    self.editor._instance.rootObject,
                    'trigger',
                    parent = object,
                    attributes = {
                      '_content_': triggertext,
                      'type': 'PRE-INSERT',
                      'name': '%s_PreInsert' % object.name
                    })

            except KeyError:
              pass


    return d  # what is returned signals the source what to do
              # with the original data (move, copy, etc.)  In this
              # case we just return the suggested value given to us.


