#
# Copyright 2001-2002 Free Software Foundation
#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# FILE:
# WidgetHighlighter.py
#
# DESCRIPTION:
#
# NOTES:

from wxPython.wx import *
import sys, string
from Utils import *
from gnue.common import GDebug


#
#
#
class WidgetHighlighter:

  def __init__(self, parent):
    self.selected = 0
    self.parent = parent
    self.x1 = 1
    self.y1 = 1
    self.x2 = 2
    self.y2 = 2
    self.west = None
    self.east = None
    self.north = None
    self.south = None
    panelColor = parent.GetBackgroundColour()
    self.inactiveBorderColor = wxColour(panelColor.Red()-16,
                 panelColor.Green()-16,
                 panelColor.Blue()-16)
    self.activeBorderColor = wxBLUE

    
  # Draw the lines comprising the highlights
  def createHighlights(self):
    if self.north:
      self.Destroy()
    self.north = wxWindow(self.parent, -1)
    self.south = wxWindow(self.parent, -1)
    self.east = wxWindow(self.parent, -1)
    self.west = wxWindow(self.parent, -1)
    for object in (self.north,self.south,self.east,self.west):
      EVT_PAINT(object, self.OnPaint)
    self.setBoundary(self.x1, self.y1, self.x2, self.y2)


  # Delete the lines comprising the highlights
  def deleteHighlights(self):
    if self.west:
      self.west.Destroy()
    if self.east:
      self.east.Destroy()
    if self.north:
      self.north.Destroy()
    if self.south:
      self.south.Destroy()
    self.west = None
    self.east = None
    self.north = None
    self.south = None


  def setBoundary(self, x1, y1, x2, y2):
     self.x1 = x1
     self.y1 = y1
     self.x2 = x2
     self.y2 = y2

     if self.west:
       self.west.SetPosition(wxPoint(x1,y1))
       self.west.SetSize(wxSize(1,y2-y1+1))
     if self.east:
       self.east.SetPosition(wxPoint(x2,y1))
       self.east.SetSize(wxSize(1,y2-y1+1))
     if self.south:
       self.south.SetPosition(wxPoint(x1,y2))
       self.south.SetSize(wxSize(x2-x1+1,1))
     if self.north:
       self.north.SetPosition(wxPoint(x1,y1))
       self.north.SetSize(wxSize(x2-x1+1,1))

  def GetChildren(self):
    if self.west:
      return (self.north, self.south, self.east, self.west)
    else:
      return ()

  def Refresh(self):
    for object in (self.north,self.south,self.east,self.west):
      object.Refresh()

  def Destroy(self):
    self.deleteHighlights()

  def setSelected (self, selected):
    if selected:
      self.selected = selected
      self.createHighlights()
      self.Refresh()
    else:
      self.deleteHighlights()

  def reconfigure(self, event):
    self.Refresh()

  def OnPaint(self, event):
    object = wxPyTypeCast(event.GetEventObject(),'wxWindow')
    dc = wxPaintDC(object)
    dc.BeginDrawing()
    dc.SetBackground(wxBrush(self.parent.GetBackgroundColour(),wxTRANSPARENT))
    dc.Clear()
    if self.selected:
      dc.SetPen(wxPen(self.activeBorderColor))
    else:
      dc.SetPen(wxPen(self.inactiveBorderColor))
    x1,y1 = object.GetSizeTuple()
    dc.DrawRectangle(0,0,x1,y1)
    dc.EndDrawing()

  def OnEraseBackground(self, event):
    pass

